@@ -1,4 +1,4 @@
-# Copyright 2001-2010, Paul Johnson (pjcj@cpan.org)
+# Copyright 2001-2011, Paul Johnson (pjcj@cpan.org)
 
 # This software is free.  It is licensed under the same terms as Perl itself.
 
@@ -10,13 +10,14 @@ package Devel::Cover;
 use strict;
 use warnings;
 
-our $VERSION = "0.73";
+our $VERSION = "0.77";
 
 use DynaLoader ();
 our @ISA = "DynaLoader";
 
-use Devel::Cover::DB  0.73;
-use Devel::Cover::Inc 0.73;
+use Devel::Cover::DB          0.77;
+use Devel::Cover::DB::Digests 0.77;
+use Devel::Cover::Inc         0.77;
 
 use B qw( class ppname main_cv main_start main_root walksymtable OPf_KIDS );
 use B::Debug;
@@ -27,6 +28,8 @@ use Config;
 use Cwd "abs_path";
 use File::Spec;
 
+use Data::Dumper; $Data::Dumper::Indent = 1; $Data::Dumper::Sortkeys = 1;
+
 BEGIN
 {
     # Use Pod::Coverage if it is available.
@@ -72,6 +75,7 @@ my $Sub_count;                           # Count for multiple subs on same line.
 
 my $Coverage;                            # Raw coverage data.
 my $Structure;                           # Structure of the files.
+my $Digests;                             # Digests of the files.
 
 my %Criteria;                            # Names of coverage criteria.
 my %Coverage;                            # Coverage criteria to collect.
@@ -107,6 +111,8 @@ BEGIN
     $^P |= 0x004 | 0x100;
 }
 
+sub version { $VERSION }
+
 if (0 && $Config{useithreads})
 {
     eval "use threads";
@@ -294,6 +300,13 @@ sub import
         warn __PACKAGE__ . ": Unknown option $_ ignored\n";
     }
 
+    if ($blib)
+    {
+        eval "use blib";
+        for (@INC) { $_ = $1 if /(.*)/ }  # Die tainting.
+        push @Ignore, "^t/", '\\.t$', '^test\\.pl$';
+    }
+
     my $ci     = $^O eq "MSWin32";
     @Select_re = map qr/$_/,                           @Select;
     @Ignore_re = map qr/$_/,                           @Ignore;
@@ -321,14 +334,7 @@ sub import
     $DB = $1 if abs_path($DB) =~ /(.*)/;
     Devel::Cover::DB->delete($DB) unless $Merge;
 
-    if ($blib)
-    {
-        eval "use blib";
-        for (@INC) { $_ = $1 if /(.*)/ }  # Die tainting.
-        push @Ignore, "^t/", '\\.t$', '^test\\.pl$';
-    }
-
-    %Files     = ();  # start gathering file information from scratch
+    %Files = ();  # start gathering file information from scratch
 
     for my $c (Devel::Cover::DB->new->criteria)
     {
@@ -413,16 +419,22 @@ sub get_coverage
 
 my %File_cache;
 
+# Recursion in normalised_file() is bad.  It can happen if a call from the sub
+# evals something which wants to load a new module.  This has happened with
+# the Storable backend.  I don't think it happens with the JSON backend.
+my $Normalising;
+
 sub normalised_file
 {
     my ($file) = @_;
 
     return $File_cache{$file} if exists $File_cache{$file};
+    return $file if $Normalising;
+    $Normalising = 1;
 
     my $f = $file;
     $file =~ s/ \(autosplit into .*\)$//;
-    # print STDERR "file is <$file>\n";
-    # use Data::Dumper;
+    $file =~ s/^\(eval in .*\) //;
     # print STDERR "file is <$file>\ncoverage: ", Dumper coverage(0);
     if (exists coverage(0)->{module} && exists coverage(0)->{module}{$file} &&
         !File::Spec->file_name_is_absolute($file))
@@ -459,8 +471,12 @@ sub normalised_file
     $file =~ s|\\|/|g if $^O eq "MSWin32";
     $file =~ s|^$Dir/|| if defined $Dir;
 
+    $Digests ||= Devel::Cover::DB::Digests->new(db => $DB);
+    $file = $Digests->canonical_file($file);
+
     # print STDERR "File: $f => $file\n";
 
+    $Normalising = 0;
     $File_cache{$f} = $file
 }
 
@@ -497,6 +513,7 @@ sub use_file
     # die "bad file" unless length $file;
 
     $file = $1 if $file =~ /^\(eval \d+\)\[(.+):\d+\]/;
+    $file = $1 if $file =~ /^\(eval in \w+\) (.+)/;
     $file =~ s/ \(autosplit into .*\)$//;
 
     return $Files{$file} if exists $Files{$file};
@@ -553,7 +570,7 @@ sub B::GV::find_cv
 sub sub_info
 {
     my ($cv) = @_;
-    my ($name, $start) = ("", 0);
+    my ($name, $start) = ("--unknown--", 0);
     if (!$cv->GV->isa("B::SPECIAL"))
     {
         return unless $cv->GV->can("SAFENAME");
@@ -649,12 +666,11 @@ sub _report
     $Structure      = Devel::Cover::DB::Structure->new(base => $DB);
     $Structure->read_all;
     $Structure->add_criteria(@collected);
-    # use Data::Dumper; $Data::Dumper::Indent = 1;
-    # use Data::Dumper; print STDERR "Start structure: ", Dumper $Structure;
+    # print STDERR "Start structure: ", Dumper $Structure;
 
     # print STDERR "Processing cover data\n@Inc\n";
     $Coverage = coverage(1) || die "No coverage data available.\n";
-    # use Data::Dumper; print STDERR Dumper $Coverage;
+    # print STDERR Dumper $Coverage;
 
     check_files();
 
@@ -699,7 +715,7 @@ sub _report
         $Structure->store_counts($file);
     }
 
-    # use Data::Dumper; print STDERR "End structure: ", Dumper $Structure;
+    # print STDERR "End structure: ", Dumper $Structure;
 
     my $run = time . ".$$." . sprintf "%05d", rand 2 ** 16;
     my $cover = Devel::Cover::DB->new
@@ -719,6 +735,7 @@ sub _report
     print OUT __PACKAGE__, ": Writing coverage database to $dbrun\n"
         unless $Silent;
     $cover->write($dbrun);
+    $Digests->write;
     $cover->print_summary if $Summary && !$Silent;
 
     return if !$Self_cover || $Self_cover_run;
@@ -837,7 +854,7 @@ sub add_condition_cover
         # if exists $condition_locations{$$op};
 
     my $key = get_key($op);
-    # print STDERR "Condition cover $$op from $File:$Line\n";
+    # warn "Condition cover $$op from $File:$Line\n";
     # print STDERR "left:  [$left]\nright: [$right]\n";
     # use Carp "cluck"; cluck("from here");
 
@@ -1102,7 +1119,7 @@ sub get_cover
 
     ($Sub_name, my $start) = sub_info($cv);
 
-    # print STDERR "get_cover: <$Sub_name>\n";
+    # warn "get_cover: <$Sub_name>\n";
     return unless defined $Sub_name;  # Only happens within Safe.pm, AFAIK.
     # return unless length  $Sub_name;  # Only happens with Self_cover, AFAIK.
 
@@ -1167,7 +1184,7 @@ sub get_cover
                 }
             }
             $Pod = "Pod::Coverage" if delete $opts{nocp};
-            # use Data::Dumper; print STDERR "$Pod, ", Dumper \%opts;
+            # print STDERR "$Pod, ", Dumper \%opts;
             if ($Pod{$file} ||= $Pod->new(package => $pkg, %opts))
             {
                 my $covered;
@@ -1295,7 +1312,7 @@ imagine that at least one of them is wrong.
 
 The most appropriate mailing list on which to discuss this module would
 be perl-qa.  Discussion has migrated there from perl-qa-metrics which is
-now defunct.  See L<http://lists.perl.org/showlist.cgi?name=perl-qa>.
+now defunct.  See L<http://lists.perl.org/list/perl-qa.html>.
 
 The Devel::Cover repository can be found at
 L<http://github.com/pjcj/Devel--Cover>.
@@ -1304,14 +1321,14 @@ L<http://github.com/pjcj/Devel--Cover>.
 
 =over
 
-=item * Perl 5.6.1 or greater.  Perl 5.8.2 or greater is recommended.
+=item * Perl 5.6.1 or greater.  Perl 5.8.8 or greater is recommended.
 
-Perl 5.7.0 is unsupported.  Perl 5.8.2 or greater is recommended.
-Whilst Perl 5.6 should mostly work you will probably miss out on
-coverage information which would be available using a more modern
-version and will likely run into bugs in perl.  Perl 5.8.0 will give
-slightly different results to more recent versions due to changes in the
-op tree.
+Perl 5.7.0 is unsupported.  Perl 5.8.8 or greater is recommended.  Perl
+5.8.7 has problems and may crash.  Whilst Perl 5.6 should mostly work
+you will probably miss out on coverage information which would be
+available using a more modern version and will likely run into bugs in
+perl.  Perl 5.8.0 will give slightly different results to more recent
+versions due to changes in the op tree.
 
 =item * The ability to compile XS extensions.
 
@@ -1500,11 +1517,11 @@ See the BUGS file.  And the TODO file.
 
 =head1 VERSION
 
-Version 0.73 - 2nd October 2010
+Version 0.77 - 15th May 2011
 
 =head1 LICENCE
 
-Copyright 2001-2010, Paul Johnson (pjcj@cpan.org)
+Copyright 2001-2011, Paul Johnson (pjcj@cpan.org)
 
 This software is free.  It is licensed under the same terms as Perl itself.
 
