@@ -1,4 +1,4 @@
-# Copyright 2001-2010, Paul Johnson (pjcj@cpan.org)
+# Copyright 2001-2011, Paul Johnson (pjcj@cpan.org)
 
 # This software is free.  It is licensed under the same terms as Perl itself.
 
@@ -10,25 +10,25 @@ package Devel::Cover::DB;
 use strict;
 use warnings;
 
-our $VERSION = "0.66";
+our $VERSION = "0.77";
 
-use Devel::Cover::Criterion     0.66;
-use Devel::Cover::DB::File      0.66;
-use Devel::Cover::DB::Structure 0.66;
+use Devel::Cover::Criterion     0.77;
+use Devel::Cover::DB::File      0.77;
+use Devel::Cover::DB::Structure 0.77;
+use Devel::Cover::DB::IO        0.77;
 
 use Carp;
 use File::Path;
-use Storable;
 
-my $DB = "cover.12";  # Version 12 of the database.
+use Data::Dumper; $Data::Dumper::Indent = 1; $Data::Dumper::Sortkeys = 1;
+
+my $DB = "cover.13";  # Version 13 of the database.
 
 @Devel::Cover::DB::Criteria =
     (qw( statement branch path condition subroutine pod time ));
 @Devel::Cover::DB::Criteria_short =
     (qw( stmt      bran   path cond      sub        pod time ));
 
-# use Data::Dumper; $Data::Dumper::Indent = 1; $Data::Dumper::Sortkeys = 1;
-
 sub new
 {
     my $class = shift;
@@ -64,9 +64,11 @@ sub all_criteria_short { @{$_[0]->{all_criteria_short}} }
 
 sub read
 {
-    my $self = shift;
-    my $file = shift;
-    my $db   = retrieve($file);
+    my $self   = shift;
+    my ($file) = @_;
+
+    my $io     = Devel::Cover::DB::IO->new;
+    my $db     = $io->read($file);
     $self->{runs} = $db->{runs};
     $self
 }
@@ -75,6 +77,7 @@ sub write
 {
     my $self = shift;
     $self->{db} = shift if @_;
+
     croak "No db specified" unless length $self->{db};
     unless (-d $self->{db})
     {
@@ -82,15 +85,10 @@ sub write
     }
     $self->validate_db;
 
-    my $db =
-    {
-        runs => $self->{runs},
-    };
-
-    Storable::nstore($db, "$self->{db}/$DB");
-
+    my $db = { runs => $self->{runs} };
+    my $io = Devel::Cover::DB::IO->new;
+    $io->write($db, "$self->{db}/$DB");
     $self->{structure}->write($self->{base}) if $self->{structure};
-
     $self
 }
 
@@ -189,9 +187,9 @@ sub is_valid
     for my $file (readdir $fh)
     {
         next if $file eq "." || $file eq "..";
-        next if ($file eq "runs" || $file eq "structure") &&
-                -e "$self->{db}/$file";
-        # warn "found $file in $self->{db}";
+        next if $file =~ /(?:runs|structure|debuglog|digests)$/
+                && -e "$self->{db}/$file";
+        warn "found $file in $self->{db}";
         return 0;
     }
     closedir $fh
@@ -208,7 +206,6 @@ sub merge
 {
     my ($self, $from) = @_;
 
-    # use Data::Dumper; $Data::Dumper::Indent = 1;
     # print STDERR "Merging ", Dumper($self), "From ", Dumper($from);
 
     while (my ($fname, $frun) = each %{$from->{runs}})
@@ -354,7 +351,7 @@ sub calculate_summary
         $self->cover->get($file)->calculate_summary($self, $file, \%options);
     }
 
-    # use Data::Dumper; print STDERR Dumper $self;
+    # print STDERR Dumper $self;
 
     for my $file ($self->cover->items)
     {
@@ -449,8 +446,8 @@ sub add_statement
         $cc->{$l}[$n][0]  += $fc->[$i];
         $cc->{$l}[$n][1] ||= $uc->{$l}[$n][0][1];
     }
-    # use Data::Dumper; print STDERR Dumper $uc;
-    # use Data::Dumper; print STDERR "cc: ", Dumper $cc;
+    # print STDERR Dumper $uc;
+    # print STDERR "cc: ", Dumper $cc;
 }
 
 sub add_time
@@ -463,7 +460,6 @@ sub add_time
         my $l = $sc->[$i];
         unless (defined $l)
         {
-            # use Data::Dumper;
             # print STDERR "sc ", scalar @$sc, ", fc ", scalar @$fc, "\n";
             # print STDERR "sc ", Dumper($sc), "fc ", Dumper($fc);
             warn "Devel::Cover: ignoring extra statement\n";
@@ -511,8 +507,7 @@ sub add_subroutine
     my $self = shift;
     my ($cc, $sc, $fc, $uc) = @_;
 
-    # use Data::Dumper;
-    #     print STDERR "add_subroutine():\n", Dumper $cc, $sc, $fc, $uc;
+    # print STDERR "add_subroutine():\n", Dumper $cc, $sc, $fc, $uc;
 
     # $cc = { line_number => [ [ count, sub_name, uncoverable ], [ ... ] ], .. }
     # $sc = [ [ line_number, sub_name ], [ ... ] ]
@@ -526,7 +521,6 @@ sub add_subroutine
         my $l = $sc->[$i][0];
         unless (defined $l)
         {
-            # use Data::Dumper;
             # print STDERR "sc ", scalar @$sc, ", fc ", scalar @$fc, "\n";
             # print STDERR "sc ", Dumper($sc), "fc ", Dumper($fc);
             warn "Devel::Cover: ignoring extra subroutine\n";
@@ -583,7 +577,7 @@ sub uncoverable
         }
     }
 
-    # use Data::Dumper; $Data::Dumper::Indent = 1; print STDERR Dumper $u;
+    # print STDERR Dumper $u;
     # Now change the format of the uncoverable information.
 
     for my $file (sort keys %$u)
@@ -605,7 +599,7 @@ sub uncoverable
         }
         close $fh;
         my $f = $u->{$file};
-        # use Data::Dumper; $Data::Dumper::Indent = 1; print STDERR Dumper $f;
+        # print STDERR Dumper $f;
         for my $crit (keys %$f)
         {
             my $c = $f->{$crit};
@@ -631,7 +625,7 @@ sub uncoverable
         $u->{$df->hexdigest} = delete $u->{$file};
     }
 
-    use Data::Dumper; $Data::Dumper::Indent = 1; print STDERR Dumper $u;
+    print STDERR Dumper $u;
     $u
 }
 
@@ -680,141 +674,84 @@ sub clean_uncoverable
     my $self = shift;
 }
 
-sub cover
+sub uncoverable_comments
 {
     my $self = shift;
-
-    return $self->{cover} if $self->{cover_valid};
-
-    my %digests;  # mapping of digests to canonical filenames
-    my %files;    # processed files
-    my $cover = $self->{cover} = {};
-
-    my $st = Devel::Cover::DB::Structure->new(base => $self->{base})->read_all;
+    my ($uncoverable, $file, $digest) = @_;
 
     my $cr = join "|", @{$self->{all_criteria}};
     my $uc = qr/(.*)# uncoverable ($cr)(.*)/;  # regex for uncoverable comments
-    my %uncoverable;
     my %types =
     (
         branch    => { true => 0, false => 1 },
         condition => { left => 0, right => 1, false => 2 },
     );
 
-    # use Data::Dumper; print STDERR "runs: ", Dumper $self->{runs};
-    my @runs;
+    # Look for uncoverable comments
+    open my $fh, "<", $file or do
     {
-        no warnings "numeric";
-        # TODO - change sort order
-        @runs = sort { $b <=> $a } keys %{$self->{runs}};
-    }
-
-    for my $run (@runs)
+        warn "Devel::Cover: Can't open file $file: $!\n";
+        next;
+    };
+    my @waiting;
+    while (<$fh>)
     {
-        my $r = $self->{runs}{$run};
-        @{$self->{collected}}{@{$r->{collected}}} = ();
-        $st->add_criteria(@{$r->{collected}});
-        my $count = $r->{count};
-        # use Data::Dumper; print STDERR "run $run, count: ", Dumper $count;
-        while (my ($file, $f) = each %$count)
+        chomp;
+        # print STDERR "read [$.][$_]\n";
+        next unless /$uc/ || @waiting;
+        if ($2)
         {
-            my $digest = $r->{digests}{$file};
-            unless ($digest)
-            {
-                print STDERR "Devel::Cover: Can't find digest for $file\n";
-                next;
-            }
-            # print STDERR "File: $file\n";
-            print STDERR "Devel::Cover: merging data for $file ",
-                         "into $digests{$digest}\n"
-                if !$files{$file}++ && $digests{$digest};
+            my ($code, $criterion, $info)     = ($1, $2, $3);
+            my ($count, $class, $note, $type) = (1, "default", "");
 
-            # Set up data structure to hold coverage being filled in
-            my $cf = $cover->{$digests{$digest} ||= $file} ||= {};
-
-            # Look for uncoverable comments
-            open my $fh, "<", $file or do
+            if ($criterion eq "branch" || $criterion eq "condition")
             {
-                warn "Devel::Cover: Can't open file $file: $!\n";
-                next;
-            };
-            my @waiting;
-            while (<$fh>)
-            {
-                chomp;
-                # print STDERR "read [$.][$_]\n";
-                next unless /$uc/ || @waiting;
-                if ($2)
+                if ($info =~ /^\s*(\w+)(?:\s|$)/)
                 {
-                    my ($code, $criterion, $info)     = ($1, $2, $3);
-                    my ($count, $class, $note, $type) = (1, "default", "");
-
-                    if ($criterion eq "branch" || $criterion eq "condition")
+                    my $t = $1;
+                    $type = $types{$criterion}{$t};
+                    unless (defined $type)
                     {
-                        if ($info =~ /^\s*(\w+)(?:\s|$)/)
-                        {
-                            my $t = $1;
-                            $type = $types{$criterion}{$t};
-                            unless (defined $type)
-                            {
-                                warn "Unknown type $t found parsing " .
-                                     "uncoverable $criterion at $file:$.\n";
-                                $type = 999;  # partly magic number
-                            }
-                        }
+                        warn "Unknown type $t found parsing " .
+                             "uncoverable $criterion at $file:$.\n";
+                        $type = 999;  # partly magic number
                     }
-                    $count = $1 if $info =~ /count:(\d+)/;
-                    $class = $1 if $info =~ /class:(\w+)/;
-                    $note  = $1 if $info =~ /note:(.+)/;
-
-                    # no warnings "uninitialized";
-                    # warn "pushing $criterion, $count, $type, $class, $note";
-
-                    push @waiting,
-                         [$criterion, $count - 1, $type, $class, $note];
-
-                    next unless $code =~ /\S/;
-                }
-
-                # found what we are waiting for
-                while (my $w = shift @waiting)
-                {
-                    my ($criterion, $count, $type, $class, $note) = @$w;
-                    push @{$uncoverable{$digest}{$criterion}{$.}[$count]},
-                         [$type, $class, $note];
                 }
             }
-            close $fh;
+            $count = $1 if $info =~ /count:(\d+)/;
+            $class = $1 if $info =~ /class:(\w+)/;
+            $note  = $1 if $info =~ /note:(.+)/;
 
-            warn scalar @waiting,
-                 " unmatched uncoverable comments not found at end of $file\n"
-                if @waiting;
+            # no warnings "uninitialized";
+            # warn "pushing $criterion, $count, $type, $class, $note";
 
-            # TODO - read in and merge $self->uncoverable;
-            # use Data::Dumper; print Dumper \%uncoverable;
+            push @waiting,
+                 [$criterion, $count - 1, $type, $class, $note];
 
-            # print STDERR "st ", Dumper($st),
-                         # "f  ", Dumper($f),
-                         # "uc ", Dumper($uncoverable{$digest});
-            while (my ($criterion, $fc) = each %$f)
-            {
-                my $get = "get_$criterion";
-                my $sc  = $st->$get($digests{$digest});
-                # print STDERR "$criterion: ", Dumper $sc, $fc;
-                next unless $sc;  # TODO - why?
-                my $cc  = $cf->{$criterion} ||= {};
-                my $add = "add_$criterion";
-                # print STDERR "$add():\n", Dumper $cc, $sc, $fc;
-                $self->$add($cc, $sc, $fc, $uncoverable{$digest}{$criterion});
-                # print STDERR "--> $add():\n", Dumper $cc;
-                # $cc - coverage being filled in
-                # $sc - structure information
-                # $fc - coverage from this file
-                # $uc - uncoverable information
-            }
+            next unless $code =~ /\S/;
+        }
+
+        # found what we are waiting for
+        while (my $w = shift @waiting)
+        {
+            my ($criterion, $count, $type, $class, $note) = @$w;
+            push @{$uncoverable->{$digest}{$criterion}{$.}[$count]},
+                 [$type, $class, $note];
         }
-        # print STDERR "Cover: ", Dumper $cover;
     }
+    close $fh;
+
+    warn scalar @waiting,
+         " unmatched uncoverable comments not found at end of $file\n"
+        if @waiting;
+
+    # TODO - read in and merge $self->uncoverable;
+    # print Dumper $uncoverable;
+}
+
+sub objectify_cover
+{
+    my $self = shift;
 
     unless (UNIVERSAL::isa($self->{cover}, "Devel::Cover::DB::Cover"))
     {
@@ -900,7 +837,80 @@ sub cover
             };
         }
     }
+}
 
+sub cover
+{
+    my $self = shift;
+
+    return $self->{cover} if $self->{cover_valid};
+
+    my %digests;  # mapping of digests to canonical filenames
+    my %files;    # processed files
+    my $cover = $self->{cover} = {};
+    my $uncoverable = {};
+    my $st = Devel::Cover::DB::Structure->new(base => $self->{base})->read_all;
+    my @runs = sort { $self->{runs}{$b}{start} <=> $self->{runs}{$a}{start} }
+                    keys %{$self->{runs}};
+    # print STDERR "runs: ", Dumper $self->{runs};
+
+    for my $run (@runs)
+    {
+        last unless $st;
+
+        my $r = $self->{runs}{$run};
+        @{$self->{collected}}{@{$r->{collected}}} = ();
+        $st->add_criteria(@{$r->{collected}});
+        my $count = $r->{count};
+        # print STDERR "run $run, count: ", Dumper $count;
+        while (my ($file, $f) = each %$count)
+        {
+            my $digest = $r->{digests}{$file};
+            unless ($digest)
+            {
+                print STDERR "Devel::Cover: Can't find digest for $file\n";
+                next;
+            }
+            # print STDERR "File: $file\n";
+            print STDERR "Devel::Cover: merging data for $file ",
+                         "into $digests{$digest}\n"
+                if !$files{$file}++ && $digests{$digest};
+
+            $self->uncoverable_comments($uncoverable, $file, $digest)
+                unless $digests{$digest};
+
+            # Set up data structure to hold coverage being filled in
+            my $cf = $cover->{$digests{$digest} ||= $file} ||= {};
+
+            # print STDERR "st ", Dumper($st),
+                         # "f  ", Dumper($f),
+                         # "uc ", Dumper($uncoverable->{$digest});
+            while (my ($criterion, $fc) = each %$f)
+            {
+                my $get = "get_$criterion";
+                my $sc  = $st->$get($digest);
+                # print STDERR "$criterion: ", Dumper $sc, $fc;
+                unless ($sc)
+                {
+                    print STDERR "Devel::Cover: Can't locate structure for ",
+                                 "$criterion in $file\n";
+                    next;
+                }
+                my $cc  = $cf->{$criterion} ||= {};
+                my $add = "add_$criterion";
+                # print STDERR "$add():\n", Dumper $cc, $sc, $fc;
+                $self->$add($cc, $sc, $fc, $uncoverable->{$digest}{$criterion});
+                # print STDERR "--> $add():\n", Dumper $cc;
+                # $cc - coverage being filled in
+                # $sc - structure information
+                # $fc - coverage from this file
+                # $uc - uncoverable information
+            }
+        }
+        # print STDERR "Cover: ", Dumper $cover;
+    }
+
+    $self->objectify_cover;
     $self->{cover_valid} = 1;
     $self->{cover}
 }
@@ -1007,11 +1017,11 @@ Huh?
 
 =head1 VERSION
 
-Version 0.66 - 12th April 2010
+Version 0.77 - 15th May 2011
 
 =head1 LICENCE
 
-Copyright 2001-2010, Paul Johnson (pjcj@cpan.org)
+Copyright 2001-2011, Paul Johnson (pjcj@cpan.org)
 
 This software is free.  It is licensed under the same terms as Perl itself.
 
