@@ -1,4 +1,4 @@
-# Copyright 2004-2010, Paul Johnson (pjcj@cpan.org)
+# Copyright 2004-2011, Paul Johnson (pjcj@cpan.org)
 
 # This software is free.  It is licensed under the same terms as Perl itself.
 
@@ -12,11 +12,16 @@ use warnings;
 
 use Carp;
 use Digest::MD5;
-use Storable;
 
 use Devel::Cover::DB;
+use Devel::Cover::DB::IO        0.77;
 
-our $VERSION = "0.73";
+use Data::Dumper; $Data::Dumper::Indent = 1; $Data::Dumper::Sortkeys = 1;
+
+# For comprehensive debug logging.
+use constant DEBUG => 0;
+
+our $VERSION = "0.77";
 our $AUTOLOAD;
 
 sub new
@@ -57,12 +62,14 @@ sub AUTOLOAD
         {
             *$func = sub
             {
-                my $self = shift;
-                my $file = shift;
-                # print STDERR "file: $file, condition: $c\n";
-                # TODO - why no file?
-                return unless defined $file;
-                $self->{f}{$file}{$c}
+                my $self   = shift;
+                my $digest = shift;
+                # print STDERR "file: $digest, condition: $c\n";
+                for my $fval (values %{$self->{f}})
+                {
+                    return $fval->{$c} if $fval->{digest} eq $digest;
+                }
+                return
             }
         };
     }
@@ -78,6 +85,24 @@ sub AUTOLOAD
     goto &$func
 }
 
+sub debuglog {
+    my $self = shift;
+    my $dir = "$self->{base}/debuglog";
+    unless (-d $dir)
+    {
+        mkdir $dir, 0700 or confess "Can't mkdir $dir: $!";
+    }
+
+    local $\;
+    # One log file per process, as we're potentially dumping out large amounts,
+    # and might excede the atomic write size of the OS.
+    open my $fh, '>>', "$dir/$$" or confess "Can't open $dir/$$: $!";
+    print $fh "----------------" . gmtime() . "----------------\n";
+    print $fh ref $_ ? Dumper($_) : $_;
+    print $fh "\n";
+    close $fh or confess "Can't close $dir/$$: $!";
+}
+
 sub add_criteria
 {
     my $self = shift;
@@ -104,7 +129,6 @@ sub set_subroutine
     # for when there are multiple subroutines of the same name on the same
     # line (such subroutines generally being called BEGIN).
 
-    # use Data::Dumper; $Data::Dumper::Indent = 1; $Data::Dumper::Sortkeys = 1;
     # print STDERR "set_subroutine start $file:$line $sub_name($scount) ",
                  # Dumper $self->{f}{$file}{start};
     $self->{additional} = 0;
@@ -165,7 +189,6 @@ sub store_counts
         $self->{f}{$file}{start}{-1}{__COVER__}[0]{$_} =
         $self->get_count($_)
         for $self->criteria;
-    # use Data::Dumper; $Data::Dumper::Indent = 1; $Data::Dumper::Sortkeys = 1;
     # print STDERR Dumper $self->{f}{$file}{start};
 }
 
@@ -225,6 +248,8 @@ sub get_count
 sub add_count
 {
     my $self = shift;
+    # warn Carp::longmess("undefined file") unless defined $self->{file};
+    return unless defined $self->{file};  # can happen during self_cover
     my ($criterion) = @_;
     $self->{additional_count}{$criterion}{$self->{file}}++
         if $self->{additional};
@@ -243,7 +268,7 @@ sub write
 {
     my $self = shift;
     my ($dir) = @_;
-    # use Data::Dumper; print STDERR Dumper $self;
+    # print STDERR Dumper $self;
     $dir .= "/structure";
     unless (-d $dir)
     {
@@ -257,11 +282,33 @@ sub write
             warn "Can't find digest for $file" unless $Devel::Cover::Silent;
             next;
         }
-        my $df = "$dir/$self->{f}{$file}{digest}";
+        my $df_final = "$dir/$self->{f}{$file}{digest}";
+        my $df_temp = "$dir/.$self->{f}{$file}{digest}.$$";
         # TODO - determine if Structure has changed to save writing it.
         # my $f = $df; my $n = 1; $df = $f . "." . $n++ while -e $df;
         # print STDERR "Writing [$file] to [$df]\n";
-        Storable::nstore($self->{f}{$file}, $df); # unless -e $df;
+        my $io = Devel::Cover::DB::IO->new;
+        $io->write($self->{f}{$file}, $df_temp); # unless -e $df;
+        unless (rename $df_temp, $df_final) {
+            unless ($Devel::Cover::Silent) {
+                if(-e $df_final) {
+                    warn "Can't rename $df_temp to $df_final (which exists): $!";
+                    $self->debuglog("Can't rename $df_temp to $df_final "
+                                    . "(which exists): $!")
+                        if DEBUG;
+                } else {
+                    warn "Can't rename $df_temp to $df_final: $!";
+                    $self->debuglog("Can't rename $df_temp to $df_final: $!")
+                        if DEBUG;
+                }
+            }
+            unless (unlink $df_temp) {
+                warn "Can't remove $df_temp after failed rename: $!"
+                    unless $Devel::Cover::Silent;
+                $self->debuglog("Can't remove $df_temp after failed rename: $!")
+                    if DEBUG;
+            }
+        }
     }
 }
 
@@ -270,10 +317,28 @@ sub read
     my $self     = shift;
     my ($digest) = @_;
     my $file     = "$self->{base}/structure/$digest";
-    my $s        = retrieve($file);
+    my $io       = Devel::Cover::DB::IO->new;
+    my $s        = eval { $io->read($file) };
+    if ($@ or !$s) {
+        $self->debuglog("read retrieve $file failed: $@") if DEBUG;
+        die $@;
+    }
+    if (DEBUG) {
+        foreach my $key (qw(file digest)) {
+            if (!defined $s->{$key}) {
+                $self->debuglog("retrieve $file had no $key entry. Got:\n", $s);
+            }
+        }
+    }
     my $d        = $self->digest($s->{file});
-    # use Data::Dumper; print STDERR "reading $digest from $file: ", Dumper $s;
-    if ($d && $d eq $s->{digest})
+    # print STDERR "reading $digest from $file: ", Dumper $s;
+    if (!$d) {
+        # No digest implies that we can't read the file. Likely this is because
+        # it's stored with a relative path. In which case, it's not valid to
+        # assume that the file has been changed, and hence that we need to
+        # "update" the structure database on disk.
+    }
+    elsif ($d eq $s->{digest})
     {
         $self->{f}{$s->{file}} = $s;
     }
@@ -281,7 +346,18 @@ sub read
     {
         warn "Devel::Cover: Deleting old coverage ",
              "for changed file $s->{file}\n";
-        unlink $file or warn "Devel::Cover: can't delete $file: $!\n";
+        if (unlink $file) {
+            $self->debuglog("Deleting old coverage $file for changed "
+                            . "$s->{file} $s->{digest} vs $d. Got:\n", $s,
+                            "Have:\n", $self->{f}{$file})
+                if DEBUG;
+        } else {
+            warn "Devel::Cover: can't delete $file: $!\n";
+            $self->debuglog("Failed to delete coverage $file for changed "
+                            . "$s->{file} ($!) $s->{digest} vs $d. Got:\n", $s,
+                            "Have:\n", $self->{f}{$file})
+                if DEBUG;
+        }
     }
     $self
 }
@@ -326,11 +402,11 @@ Huh?
 
 =head1 VERSION
 
-Version 0.73 - 2nd October 2010
+Version 0.77 - 15th May 2011
 
 =head1 LICENCE
 
-Copyright 2004-2010, Paul Johnson (pjcj@cpan.org)
+Copyright 2004-2011, Paul Johnson (pjcj@cpan.org)
 
 This software is free.  It is licensed under the same terms as Perl itself.
 
