@@ -1,4 +1,4 @@
-# Copyright 2001-2010, Paul Johnson (pjcj@cpan.org)
+# Copyright 2001-2011, Paul Johnson (pjcj@cpan.org)
 
 # This software is free.  It is licensed under the same terms as Perl itself.
 
@@ -10,10 +10,10 @@ package Devel::Cover::Report::Html_basic;
 use strict;
 use warnings;
 
-our $VERSION = "0.66";
+our $VERSION = "0.77";
 
-use Devel::Cover::DB 0.66;
-use Devel::Cover::Web 0.66 "write_file";
+use Devel::Cover::DB 0.77;
+use Devel::Cover::Web 0.77 "write_file";
 
 use Getopt::Long;
 use Template 2.00;
@@ -42,6 +42,7 @@ sub class
 {
     my ($pc, $err, $criterion) = @_;
     return "" if $criterion eq "time";
+    no warnings "uninitialized";
     !$err ? "c3"
           : $pc <  75 ? "c0"
           : $pc <  90 ? "c1"
@@ -448,7 +449,7 @@ package Devel::Cover::Report::Html_basic::Template::Provider;
 use strict;
 use warnings;
 
-our $VERSION = "0.66";
+our $VERSION = "0.77";
 
 use base "Template::Provider";
 
@@ -468,8 +469,8 @@ $Templates{html} = <<'EOT';
      "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
 <html xmlns="http://www.w3.org/1999/xhtml">
 <!--
-This file was generated by Devel::Cover Version 0.66
-Devel::Cover is copyright 2001-2010, Paul Johnson (pjcj@cpan.org)
+This file was generated by Devel::Cover Version 0.77
+Devel::Cover is copyright 2001-2011, Paul Johnson (pjcj@cpan.org)
 Devel::Cover is free. It is licensed under the same terms as Perl itself.
 The latest version of Devel::Cover should be available from my homepage:
 http://www.pjcj.net
@@ -769,11 +770,11 @@ Huh?
 
 =head1 VERSION
 
-Version 0.66 - 12th April 2010
+Version 0.77 - 15th May 2011
 
 =head1 LICENCE
 
-Copyright 2001-2010, Paul Johnson (pjcj@cpan.org)
+Copyright 2001-2011, Paul Johnson (pjcj@cpan.org)
 
 This software is free.  It is licensed under the same terms as Perl itself.
 
